/// <reference types="mqtt" />
import { PubSubEngine } from 'graphql-subscriptions/dist/pubsub';
import { Client, ClientPublishOptions, ClientSubscribeOptions, Granted } from 'mqtt';
export interface PubSubMQTTOptions {
    brokerUrl?: string;
    client?: Client;
    connectionListener?: (err: Error) => void;
    publishOptions?: PublishOptionsResolver;
    subscribeOptions?: SubscribeOptionsResolver;
    onMQTTSubscribe?: (id: number, granted: Granted) => void;
    triggerTransform?: TriggerTransform;
    parseMessageWithEncoding?: string;
}
export declare class MQTTPubSub implements PubSubEngine {
    constructor(options?: PubSubMQTTOptions);
    publish(trigger: string, payload: any): boolean;
    subscribe(trigger: string, onMessage: Function, options?: Object): Promise<number>;
    unsubscribe(subId: number): void;
    private onMessage(topic, message);
    private triggerTransform;
    private onMQTTSubscribe;
    private subscribeOptionsResolver;
    private publishOptionsResolver;
    private mqttConnection;
    private subscriptionMap;
    private subsRefsMap;
    private currentSubscriptionId;
    private parseMessageWithEncoding;
}
export declare type Path = Array<string | number>;
export declare type Trigger = string | Path;
export declare type TriggerTransform = (trigger: Trigger, channelOptions?: Object) => string;
export declare type SubscribeOptionsResolver = (trigger: Trigger, channelOptions?: Object) => Promise<ClientSubscribeOptions>;
export declare type PublishOptionsResolver = (trigger: Trigger, payload: any, channelOptions?: Object) => Promise<ClientPublishOptions>;
export declare type SubscribeHandler = (id: number, granted: Granted) => void;
